###########################################################################
# TextIndexNG V 3
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

"""
A collection of adapter to provide out-of-the-box indexing support
for Plone content-types.

$Id: plone_adapters.py 2166 2009-04-22 19:53:38Z yvoschu $
"""

from zope.component import adapts

from Products.ATContentTypes.interface.file import IATFile

from cmf_adapters import CMFContentAdapter


class ATFileAdapter(CMFContentAdapter):

    """An adapter for ATCT files.
    """

    adapts(IATFile)

    def addSearchableTextField(self, icc):
        text = self._c(self.context.SearchableText())
        icc.addContent('SearchableText', text, self.language)

        f = self.context.getFile()
        if not f:
            return

        body = str(f)
        if body:
            mt = f.getContentType()
            if mt == 'text/plain':
                icc.addContent('SearchableText', self._c(body), self.language)
            else:
                icc.addBinary('SearchableText', body, mt, None, self.language)



###########################################################################
# TextIndexNG V 3                
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################


import os, sys

try:
    import zopyx
except ImportError:
    raise ImportError('The zopyx package could not be imported. Please check the installation of the TextIndexNG extension modules!')

package_home = os.path.dirname(__file__)

# add base package to sys.path
src_dir = os.path.abspath(os.path.join(package_home, 'src'))
if not src_dir in sys.path:
    sys.path.append(src_dir)


import TextIndexNG3

def initialize(context):
    context.registerClass(
        TextIndexNG3.TextIndexNG3,
        permission='Add Pluggable Index',
        icon='pt/index.gif',
        visibility=None,
        constructors=(TextIndexNG3.manage_addTextIndexNG3Form,
                      TextIndexNG3.manage_addTextIndexNG3),
    )


def get_storages(self):
    """ return all available storages """
    from zope.app import zapi
    from zopyx.txng3.core.interfaces import IStorage, IStorageWithTermFrequency
    return [x[0] for x in zapi.getUtilitiesFor(IStorage)]

def get_lexicons(self):
    """ return all available lexicons"""
    from zope.app import zapi
    from zopyx.txng3.core.interfaces import ILexicon
    return [x[0] for x in zapi.getUtilitiesFor(ILexicon)]

def get_splitters(self):
    """ return all available splitters"""
    from zope.app import zapi
    from zopyx.txng3.core.interfaces import ISplitter
    return [x[0] for x in zapi.getUtilitiesFor(ISplitter)]

def get_parsers(self):
    """ return all available parsers"""
    from zope.app import zapi
    from zopyx.txng3.core.interfaces import IParser
    return [x[0] for x in zapi.getUtilitiesFor(IParser)]

# some monkey patching (necessary since we have no context object
# during the factory phase where we could use browser views)

from Products.ZCatalog.ZCatalog import ZCatalog
ZCatalog.get_storages = get_storages
ZCatalog.get_lexicons = get_lexicons
ZCatalog.get_splitters = get_splitters       
ZCatalog.get_parsers = get_parsers

# Plone
textindexng_globals = globals()

try:
    from Products.CMFCore.DirectoryView import registerDirectory
    registerDirectory('skins', textindexng_globals)
except ImportError:
    pass



###########################################################################
# TextIndexNG V 3
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

"""
TextIndexNG3 unit tests

$Id: testTextIndexNG3.py 2140 2009-03-15 14:42:18Z ajung $
"""

import sys, os, unittest
import txngtest
from Testing import ZopeTestCase
ZopeTestCase.installProduct('ZCatalog', 1)
ZopeTestCase.installProduct('TextIndexNG3', 1)

from zope.interface.verify import verifyClass
from OFS.DTMLDocument import addDTMLDocument

from Products.TextIndexNG3 import package_home
from Products.TextIndexNG3.TextIndexNG3 import TextIndexNG3

from zopyx.txng3.core.config import *
from zopyx.txng3.core.resultset import ResultSet


class TextIndexNG3Tests(ZopeTestCase.ZopeTestCase):

    layer = txngtest.TextIndexNG3ZCMLLayer

    def afterSetUp(self):
        factory = self.folder.manage_addProduct['ZCatalog']
        factory.manage_addZCatalog('catalog', 'catalog')
        catalog = self.folder['catalog']

    def testInterfaceMock(self):
        from Products.PluginIndexes.interfaces import IPluggableIndex
        verifyClass(IPluggableIndex, TextIndexNG3)

    def testSimpleSetup(self):
        extra = {'lexicon' : DEFAULT_LEXICON,
                 'storage' : DEFAULT_STORAGE,
                 'splitter' : DEFAULT_SPLITTER}
        self.folder.catalog.manage_addIndex('PrincipiaSearchSource', 'TextIndexNG3', extra)

    def testStemmer(self):
        extra = {'use_stemmer' : True}
        self.folder.catalog.manage_addIndex('PrincipiaSearchSource', 'TextIndexNG3', extra)
        idx = self.folder.catalog.Indexes['PrincipiaSearchSource']
        self.assertEqual(idx.index.use_stemmer, True)

        extra = {'use_stemmer' : False}
        self.folder.catalog.manage_addIndex('PrincipiaSearchSource1', 'TextIndexNG3', extra)
        idx = self.folder.catalog.Indexes['PrincipiaSearchSource1']
        self.assertEqual(idx.index.use_stemmer, False)

    def testIndexing(self):
        extra = {'lexicon' : DEFAULT_LEXICON,
                 'storage' : DEFAULT_STORAGE,
                 'splitter' : DEFAULT_SPLITTER}
        self.folder.catalog.manage_addIndex('PrincipiaSearchSource', 'TextIndexNG3', extra)

        from zopyx.txng3.core import tests
        datadir = os.path.join(os.path.dirname(tests.__file__), 'data', 'texts')
        for f in os.listdir(datadir):
            fname = os.path.join(datadir, f)
            if not os.path.isfile(fname): continue
            fp = open(fname)
            addDTMLDocument(self.folder, id=f, title=f, file=fp)
            fp.close()

        for obj in [o for o in self.folder.objectValues('DTML Document') if o.getId().endswith('txt')]:
            self.folder.catalog.catalog_object(obj, obj.absolute_url(1))
        self.assertEqual(len(self.folder.catalog), 199)

    def test_apply_index(self):
        self.folder.catalog.manage_addIndex('foo', 'TextIndexNG3')
        idx = self.folder.catalog.Indexes['foo']
        def dummysearch(query, **kw):
            return ResultSet((query, kw), None)
        idx.index.search = dummysearch

        request = {}
        self.assertEqual(idx._apply_index(request), None)

        request = {'foo': 'Foo'}
        self.assertEqual(idx._apply_index(request)[0][0], u'Foo')
        self.assertEqual(idx._apply_index(request)[0][1], {})

        request = {'foo': {'query': 'Foo'}}
        self.assertEqual(idx._apply_index(request)[0][0], u'Foo')
        self.assertEqual(idx._apply_index(request)[0][1], {})

        request = {'foo': {'query': 'Foo', 'parser': 'foo_parser'}}
        self.assertEqual(idx._apply_index(request)[0][0], u'Foo')
        self.assertEqual(idx._apply_index(request)[0][1],
                         {'parser': 'foo_parser'})

        request = {'foo': {'query': 'Foo', 'ranking': False}}
        self.assertEqual(idx._apply_index(request)[0][0], u'Foo')
        self.assertEqual(idx._apply_index(request)[0][1],
                         {'ranking': False})


def test_suite():
    s = unittest.TestSuite()
    s.addTest(unittest.makeSuite(TextIndexNG3Tests))
    return s

def main():
    unittest.TextTestRunner().run(test_suite())

def debug():
    test_suite().debug()

def pdebug():
    import pdb
    pdb.run('debug()')

if __name__=='__main__':
    if len(sys.argv) > 1:
        globals()[sys.argv[1]]()
    else:
        main()



# -*- coding: iso-8859-15 -*-

###########################################################################
# TextIndexNG V 3
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

"""
adapter unit tests

$Id: testAdapters.py 2165 2009-04-22 19:10:10Z yvoschu $
"""

import unittest
import os
import sys
import txngtest
from Testing import ZopeTestCase
ZopeTestCase.installProduct('ZCatalog', 1)
ZopeTestCase.installProduct('TextIndexNG3', 1)

from zopyx.txng3.core import tests

try:
    from Products import CMFDefault
    del CMFDefault
    _CMF_INSTALLED = True
except ImportError:
    _CMF_INSTALLED = False

_DATA_DIR = os.path.join(os.path.dirname(tests.__file__), 'data')


class CMFDocumentAdapterTests(unittest.TestCase):

    layer = txngtest.TextIndexNG3ZCMLLayer

    def _getTargetClass(self):
        from Products.TextIndexNG3.adapters.cmf_adapters import CMFDocumentAdapter

        return CMFDocumentAdapter

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_indexableContent_plain(self):
        from Products.CMFCore.CatalogTool import IndexableObjectWrapper
        from Products.CMFDefault.Document import Document

        d = Document('foo', title='Foo', text_format='plain',
                     text='foo content')
        d.setLanguage('en')
        try:
            w = IndexableObjectWrapper(ob=d, catalog=None)
        except TypeError:
            w = IndexableObjectWrapper({}, d) # BBB: for CMF < 2.2
        a = self._makeOne(w)

        icc = a.indexableContent(['Title', 'Description'])
        self.assertEqual(icc.getFields(), ['Description', 'Title'])
        self.assertEqual(icc.getFieldData('Title')[0],
                         {'content': u'Foo', 'language': 'en'})
        self.assertEqual(icc.getFieldData('Description')[0],
                         {'content': u'', 'language': 'en'})

        icc = a.indexableContent(['SearchableText'])
        self.assertEqual(icc.getFields(), ['SearchableText'])
        self.assertEqual(icc.getFieldData('SearchableText')[0],
                         {'content': u'Foo', 'language': 'en'})
        self.assertEqual(icc.getFieldData('SearchableText')[1],
                         {'content': u'', 'language': 'en'})
        self.assertEqual(icc.getFieldData('SearchableText')[2],
                         {'content': u'foo content', 'language': 'en'})

    def test_indexableContent_html(self):
        from Products.CMFCore.CatalogTool import IndexableObjectWrapper
        from Products.CMFDefault.Document import Document

        d = Document('foo', title='Foo', text_format='html',
                     text='foo content')
        d.setLanguage('en')
        try:
            w = IndexableObjectWrapper(ob=d, catalog=None)
        except TypeError:
            w = IndexableObjectWrapper({}, d) # BBB: for CMF < 2.2
        a = self._makeOne(w)

        icc = a.indexableContent(['Title', 'Description'])
        self.assertEqual(icc.getFields(), ['Description', 'Title'])
        self.assertEqual(icc.getFieldData('Title')[0],
                         {'content': u'Foo', 'language': 'en'})
        self.assertEqual(icc.getFieldData('Description')[0],
                         {'content': u'', 'language': 'en'})

        icc = a.indexableContent(['SearchableText'])
        self.assertEqual(icc.getFields(), ['SearchableText'])
        self.assertEqual(icc.getFieldData('SearchableText')[0],
                         {'content': u'Foo', 'language': 'en'})
        self.assertEqual(icc.getFieldData('SearchableText')[1],
                         {'content': u'', 'language': 'en'})
        self.assertEqual(icc.getFieldData('SearchableText')[2],
                         {'content': u'foo content', 'language': 'en'})


class CMFFileAdapterTests(unittest.TestCase):

    layer = txngtest.TextIndexNG3ZCMLLayer

    def _getTargetClass(self):
        from Products.TextIndexNG3.adapters.cmf_adapters import CMFFileAdapter

        return CMFFileAdapter

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def test_indexableContent_plain(self):
        from Products.CMFCore.CatalogTool import IndexableObjectWrapper
        from Products.CMFDefault.File import File

        d = File('foo', title='Foo', file='foo content',
                 content_type='text/plain')
        d.setLanguage('en')
        try:
            w = IndexableObjectWrapper(ob=d, catalog=None)
        except TypeError:
            w = IndexableObjectWrapper({}, d) # BBB: for CMF < 2.2
        a = self._makeOne(w)

        icc = a.indexableContent(['Title', 'Description'])
        self.assertEqual(icc.getFields(), ['Description', 'Title'])
        self.assertEqual(icc.getFieldData('Title')[0],
                         {'content': u'Foo', 'language': 'en'})
        self.assertEqual(icc.getFieldData('Description')[0],
                         {'content': u'', 'language': 'en'})

        icc = a.indexableContent(['SearchableText'])
        self.assertEqual(icc.getFields(), ['SearchableText'])
        self.assertEqual(icc.getFieldData('SearchableText')[0],
                         {'content': u'Foo', 'language': 'en'})
        self.assertEqual(icc.getFieldData('SearchableText')[1],
                         {'content': u'', 'language': 'en'})
        self.assertEqual(icc.getFieldData('SearchableText')[2],
                         {'content': u'foo content', 'language': 'en'})

    def test_indexableContent_pdf(self):
        from Products.CMFCore.CatalogTool import IndexableObjectWrapper
        from Products.CMFDefault.File import File

        pdf_path = os.path.join(_DATA_DIR, 'test.pdf')
        d = File('foo', title='Foo', file=file(pdf_path, 'rb'),
                 content_type='application/pdf')
        d.setLanguage('de')
        try:
            w = IndexableObjectWrapper(ob=d, catalog=None)
        except TypeError:
            w = IndexableObjectWrapper({}, d) # BBB: for CMF < 2.2
        a = self._makeOne(w)

        icc = a.indexableContent(['Title', 'Description'])
        self.assertEqual(icc.getFields(), ['Description', 'Title'])
        self.assertEqual(icc.getFieldData('Title')[0],
                         {'content': u'Foo', 'language': 'de'})
        self.assertEqual(icc.getFieldData('Description')[0],
                         {'content': u'', 'language': 'de'})

        icc = a.indexableContent(['SearchableText'])
        self.assertEqual(icc.getFields(), ['SearchableText'])
        self.assertEqual(icc.getFieldData('SearchableText')[0],
                         {'content': u'Foo', 'language': 'de'})
        self.assertEqual(icc.getFieldData('SearchableText')[1],
                         {'content': u'', 'language': 'de'})
        body = icc.getFieldData('SearchableText')[2]
        self.assertEqual(body['content'].strip(), u'Viel Vögel sprangen artig '
                         u'in den Tüpel und über Feld und Wüste')
        self.assertEqual(body['language'], 'de')


def test_suite():
    s = unittest.TestSuite()
    if _CMF_INSTALLED:
        s.addTest(unittest.makeSuite(CMFDocumentAdapterTests))
        s.addTest(unittest.makeSuite(CMFFileAdapterTests))
    else:
        print 'Skipped CMF adapter tests.'
    return s

def main():
    unittest.TextTestRunner().run(test_suite())

def debug():
    test_suite().debug()

def pdebug():
    import pdb
    pdb.run('debug()')

if __name__=='__main__':
    if len(sys.argv) > 1:
        globals()[sys.argv[1]]()
    else:
        main()



# -*- coding: iso-8859-15 -*-

################################################################
# SRMedia
#
# (C) 2006 
# ZOPYX Limited & Co. KG
# Charlottenstr. 37/1, D-72070 Tübingen, Germany
# info@zopyx.com, www.zopyx.com 
################################################################

import unittest

try:
    from Products.PloneTestCase import PloneTestCase
    PloneTestCase.installProduct('Five')
    PloneTestCase.installProduct('CMFPlone')
    PloneTestCase.installProduct('TextIndexNG3')

    # setup a new Plohn site    
    PloneTestCase.setupPloneSite(products=('TextIndexNG3', ))
    PTC = PloneTestCase.PloneTestCase
    _PLONE_INSTALLED = True
except ImportError:
    PTC = object
    _PLONE_INSTALLED = False


class PloneTests(PTC):

    def afterSetUp(self):
        membership = self.portal.portal_membership
        membership.addMember('god', 'god', ('Manager',), ())

        self.login('god')
        self.portal.txng_convert_indexes()

    def testSetup(self):
        c = self.portal.portal_catalog
        indexes = c.Indexes
        self.assertEqual(indexes['SearchableText'].meta_type, 'TextIndexNG3')
        self.assertEqual(indexes['Title'].meta_type, 'TextIndexNG3')


def test_suite():
    s = unittest.TestSuite()
    if _PLONE_INSTALLED:
        s.addTest(unittest.makeSuite(PloneTests))
    else:
        print 'Skipped Plone setup tests.'
    return s



###########################################################################
# TextIndexNG V 3                
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

from zope.interface import Interface

class ITextIndexNG3(Interface):
    """ """
    pass



##parameters=overwrite=0

###########################################################################
# TextIndexNG V 3                
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

# Replace all text indexes with a TextIndexNG V3 instance

# set language as appropriate
LANGUAGES = ('en', )
#LANGUAGES = ('en', 'de')

encoding = context.portal_properties.site_properties.default_charset

metatypes =  ['ZCTextIndex', 'TextIndex', 'TextIndexNG2', 'TextIndexNG3']
needed = ('SearchableText', 'Title', 'Description')

catalog = context.portal_catalog
indexes = catalog.getIndexObjects()
all_ids = [idx.getId() for idx in indexes if idx.meta_type in metatypes]
existing_ids = all_ids[:]

for id in needed:
    if not id in all_ids:
        all_ids.append(id)

for id in all_ids:

    if id in existing_ids:
        catalog.manage_delIndex(id)
    catalog.manage_addIndex(id, 'TextIndexNG3', extra={'languages': LANGUAGES,
                                                           'default_encoding' : encoding,
                                                           'splitter_casefolding' : True,
                                                           'dedicated_storages' : True,
                                                           'use_converters' : True,
                                                           'index_unknown_languages' : True,
                                                           'storage' : 'txng.storages.term_frequencies',
                                                           'ranking' : True,
                                                           })

catalog.manage_reindexIndex(all_ids, context.REQUEST)
context.REQUEST.RESPONSE.redirect('txng_maintenance?portal_status_message=All+TXNG+indexes+migrated+and+fixed')



###########################################################################
# TextIndexNG V 3
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

"""
A collection of adapter to provide better indexing support for
CMF derived types.

$Id: cmf_adapters.py 2168 2009-04-22 20:10:39Z yvoschu $
"""

from zope.component import adapts
from zope.interface import implements

from Products.CMFCore.interfaces import IContentish
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.interfaces import IDocument
from Products.CMFDefault.interfaces import IFile

from zopyx.txng3.core.interfaces import IIndexableContent
from zopyx.txng3.core.content import IndexContentCollector as ICC
from zopyx.txng3.core.logger import LOG
from zopyx.txng3.core.config import DEFAULT_LANGUAGE


class CMFContentAdapter:

    """An adapter for CMF content.
    """

    adapts(IContentish)
    implements(IIndexableContent)

    def __init__(self, context):
        self.context = context
        self.encoding = self._getSiteEncoding()
        try:
            self.language = context.Language()
        except AttributeError:
            self.language = DEFAULT_LANGUAGE

    def _getSiteEncoding(self):
        plone_utils = getToolByName(self.context, 'plone_utils', None)
        if plone_utils is not None:
            return plone_utils.getSiteEncoding()

        ptool = getToolByName(self.context, 'portal_properties', None)
        if ptool is not None:
            return ptool.getProperty('default_charset', 'utf-8')

        return 'utf-8'

    def _c(self, text):
        if isinstance(text, unicode):
            return text
        try:
            return unicode(text, self.encoding)
        except UnicodeDecodeError:
            LOG.warn('Content from %s could not be converted to unicode using the site encoding %s' %
                    (self.context.absolute_url(1), self.encoding))
            raise

    def indexableContent(self, fields):
        icc = ICC()
        if 'getId' in fields:
            self.addIdField(icc)
        if 'Title' in fields:
            self.addTitleField(icc)
        if 'Description' in fields:
            self.addDescriptionField(icc)
        if 'SearchableText' in fields:
            self.addSearchableTextField(icc)
        return icc

    def addIdField(self, icc):
        id = self._c(self.context.getId())
        icc.addContent('getId', id, self.language)

    def addTitleField(self, icc):
        title = self._c(self.context.Title())
        icc.addContent('Title', title, self.language)

    def addDescriptionField(self, icc):
        description = self._c(self.context.Description())
        icc.addContent('Description', description, self.language)

    def addSearchableTextField(self, icc):
        text = self._c(self.context.SearchableText())
        icc.addContent('SearchableText', text, self.language)


class CMFDocumentAdapter(CMFContentAdapter):

    """An adapter for CMFDefault documents.
    """

    adapts(IDocument)

    def addSearchableTextField(self, icc):
        title = self._c(self.context.Title())
        icc.addContent('SearchableText', title, self.language)

        description = self._c(self.context.Description())
        icc.addContent('SearchableText', description, self.language)

        body = self._c(self.context.EditableBody())
        if body:
            mt = self.context.Format()
            if mt == 'text/plain':
                icc.addContent('SearchableText', body, self.language)
            else:
                icc.addBinary('SearchableText', body, mt, None, self.language)


class CMFFileAdapter(CMFContentAdapter):

    """An adapter for CMFDefault files.
    """

    adapts(IFile)

    def addSearchableTextField(self, icc):
        title = self._c(self.context.Title())
        icc.addContent('SearchableText', title, self.language)

        description = self._c(self.context.Description())
        icc.addContent('SearchableText', description, self.language)

        body = str(self.context)
        if body:
            mt = self.context.Format()
            if mt == 'text/plain':
                icc.addContent('SearchableText', self._c(body), self.language)
            else:
                icc.addBinary('SearchableText', body, mt, None, self.language)



###########################################################################
# TextIndexNG V 3
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

"""
export / import support unit tests

$Id: testExportImport.py 1996 2008-11-04 18:21:50Z yvoschu $
"""

import unittest
import sys
import txngtest

try:
    from Products.GenericSetup.testing import NodeAdapterTestCase
    _GS_INSTALLED = True
except ImportError:
    class NodeAdapterTestCase: pass
    _GS_INSTALLED = False

_TXNG3_XML = """\
<index name="foo_txng3" meta_type="TextIndexNG3">
 <autoexpand value="off"/>
 <autoexpand_limit value="4"/>
 <dedicated_storage value="True"/>
 <default_encoding value="iso-8859-15"/>
 <field value="foo_txng3"/>
 <index_unknown_languages value="True"/>
 <language value="en"/>
 <lexicon value="txng.lexicons.default"/>
 <query_parser value="txng.parsers.en"/>
 <ranking value="False"/>
 <splitter value="txng.splitters.simple"/>
 <splitter_additional_chars value="_-"/>
 <splitter_casefolding value="True"/>
 <storage value="txng.storages.default"/>
 <use_normalizer value="False"/>
 <use_stemmer value="False"/>
 <use_stopwords value="False"/>
</index>
"""


class TextIndexNG3NodeAdapterTests(NodeAdapterTestCase, unittest.TestCase):

    layer = txngtest.TextIndexNG3ZCMLLayer

    def _getTargetClass(self):
        from Products.TextIndexNG3.exportimport import TextIndexNG3NodeAdapter

        return TextIndexNG3NodeAdapter

    def _populate(self, obj):
        obj.index.__init__(fields=('foo_txng3',))

    def setUp(self):
        from Products.TextIndexNG3.TextIndexNG3 import TextIndexNG3

        self._obj = TextIndexNG3('foo_txng3', None, None)
        self._XML = _TXNG3_XML


def test_suite():
    s = unittest.TestSuite()
    if _GS_INSTALLED:
        s.addTest(unittest.makeSuite(TextIndexNG3NodeAdapterTests))
    else:
        print 'Skipped GenericSetup export/import tests.'
    return s

def main():
    unittest.TextTestRunner().run(test_suite())

def debug():
    test_suite().debug()

def pdebug():
    import pdb
    pdb.run('debug()')

if __name__=='__main__':
    if len(sys.argv) > 1:
        globals()[sys.argv[1]]()
    else:
        main()



###########################################################################
# TextIndexNG V 3                
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

try:
    from App.class_init import InitializeClass
except ImportError:
    from Globals import InitializeClass #BBB: for Zope < 2.11.3

from Products.Five import BrowserView


class IndexView(BrowserView):

    def test_index(self, query, parser, encoding='iso-8859-15'):
        """ perform a query and return the search result as list of
            object paths
        """
        if not isinstance(query, unicode):
            pattern = unicode(pattern, encoding)
        rs = self.context.index.search(query, parser=parser)
        return [self.context.getpath(docid) for docid in rs.getDocids()]

    def words_vocabulary(self, pattern, language='en'):
        """ return all words from the lexicon that match a particualar
            pattern for a given language.
        """
        if not isinstance(pattern, unicode):
            pattern = unicode(pattern, 'utf-8') # ZMI encoding
        words = self.context.index.getLexicon().getWordsForPattern(pattern, language)
        words.sort()
        return words

    def get_vocabulary_stats(self):
        """ return per-language length of the vocabulary """
        lexicon = self.context.index.getLexicon()
        d = dict()
        for lang in lexicon.getLanguages():
            d[lang] = len(lexicon.getWordsForLanguage(lang))
        return d

    def _getUtilitiesFor(self, iface):
        """ return everything registered for an interface """
        from zope.app import zapi
        return zapi.getUtilitiesFor(iface)

    def get_converters(self):
        """ return all available converters """
        from zopyx.txng3.core.interfaces import IConverter
        return self._getUtilitiesFor(IConverter)

    def get_storages(self):
        """ return all available storages """
        from zopyx.txng3.core.interfaces import IStorage
        return self._getUtilitiesFor(IStorage)

    def get_lexicons(self):
        """ return all available lexicons """
        from zopyx.txng3.core.interfaces import ILexicon
        return self._getUtilitiesFor(ILexicon)

    def get_thesauruses(self):
        """ return all available thesaurus"""
        from zopyx.txng3.core.interfaces import IThesaurus
        return self._getUtilitiesFor(IThesaurus)

    def get_thesaurus(self):
        """ return the content for a particular thesurus """
        from zope.app import zapi
        from zopyx.txng3.core.interfaces import IThesaurus
        return zapi.getUtility(IThesaurus, self.request['id'])

    def get_splitters(self):
        """ return all available lexicons """
        from zopyx.txng3.core.interfaces import ISplitter
        return self._getUtilitiesFor(ISplitter)

    def get_parsers(self):
        """ return all available lexicons """
        from zopyx.txng3.core.interfaces import IParser
        return self._getUtilitiesFor(IParser)

    def documents_for_word(self):
        """ return all document paths for a given word """
        word = self.request['word']
        language = self.request.get('language', self.context.index.languages[0])
        if not isinstance(word, unicode):
            word = unicode(word, 'utf-8') # ZMI encoding
        wid = self.context.index.getLexicon().getWordId(word,language)
        docids = self.context.index.getStorage(self.context.index.fields[0]).getDocumentsForWordId(wid)
        return [self.context.getpath(docid) for docid in docids]

    def get_adapters(self):
        """ return all registered adapters"""

        from zopyx.txng3.core.interfaces import IIndexableContent
        from zope.component.globalregistry import getGlobalSiteManager as getSiteManager

        registrations = getSiteManager().registeredAdapters()

        for reg in registrations:
            if reg.provided == IIndexableContent:
                yield (reg.required, reg.factory)

InitializeClass(IndexView)



###########################################################################
# TextIndexNG V 3                
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

from StringIO import StringIO

import transaction
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.TextIndexNG3 import textindexng_globals

key = 'textindexng3'

configlets = \
( { 'id'         : 'TextIndexNG3'
  , 'name'       : 'TextIndexNG3'
  , 'action'     : 'string:${portal_url}/txng_maintenance'
  , 'category'   : 'Products'
  , 'appId'      : 'TextIndexNG3'
  , 'permission' : ManagePortal
  , 'imageUrl'   : 'index.gif'
  }
,
)

def install(self):
    out = StringIO()

    skins_tool = getToolByName(self, 'portal_skins')

    # Setup the skins
    if key  not in skins_tool.objectIds():
        addDirectoryViews(skins_tool, 'skins',  textindexng_globals)
        out.write("Added %s skin directory view to portal_skins\n" % key)

    # Now we need to go through the skin configurations and insert
    # 'textindexng' into the configurations.  Preferably, this should be
    # right before where 'content' is placed.  Otherwise, we append
    # it to the end.
    skins = skins_tool.getSkinSelections()

    for skin in skins:
        path = skins_tool.getSkinPath(skin)
        path = [p.strip() for p in  path.split(',')]

        if key not in path:
            try: path.insert(0, key)
            except ValueError:
                path.append(key)

            path = ', '.join(path)
            # addSkinSelection will replace exissting skins as well.
            skins_tool.addSkinSelection(skin, path)
            out.write("Added '%s' to %s skin\n" % (key, skin))
        else:
            out.write("Skipping %s skin, '%s' is already set up\n" % (
                key, skin))


    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in configlets:
            print >>out, 'Adding configlet %s\n' % conf['id']
            configTool.registerConfiglet(**conf)

    print >> out, "Successfully installed"  
    return out.getvalue()


def uninstall(self, reinstall=False):
    out = StringIO()

    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        for conf in configlets:
            print >>out, 'Removing configlet %s\n' % conf['id']
            configTool.unregisterConfiglet(conf['id'])

    skins_tool = getToolByName(self, 'portal_skins')

    # Now we need to go through the skin configurations and insert
    # 'textindexng3' into the configurations.  Preferably, this should be
    # right before where 'content' is placed.  Otherwise, we append
    # it to the end.
    skins = skins_tool.getSkinSelections()

    for skin in skins:
        path = skins_tool.getSkinPath(skin)
        path = [p.strip() for p in  path.split(',')]

        if key in path:
            path.remove(key)

            path = ', '.join(path)
            # addSkinSelection will replace exissting skins as well.
            skins_tool.addSkinSelection(skin, path)
            out.write("Removed '%s' from %s skin\n" % (key, skin))
        else:
            out.write("Skipping %s skin, '%s' is removed set up\n" % (
                key, skin))

    if not reinstall:
        # uninstall TXGN3 indexes and re-create the original TextIndexes

        catalog = getToolByName(self, 'portal_catalog')
        indexes = catalog.getIndexObjects()
        indexes = [idx.getId() for idx in indexes
                    if idx.meta_type in ('TextIndexNG3',)]

        class _extra:
            lexicon_id="plone_lexicon"
            index_type="Okapi BM25 Rank"
        extra=_extra()

        for idx in indexes:
            catalog.manage_delIndex(idx)
            catalog.addIndex(idx, "ZCTextIndex", extra=extra)
            index=catalog._catalog.getIndex(idx)
            index.indexed_attrs=[idx]

        catalog.manage_reindexIndex(indexes, self.REQUEST)

    return out.getvalue()



###########################################################################
# TextIndexNG V 3
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

"""
TextIndexNG3

$Id: TextIndexNG3.py 2124 2009-03-15 12:42:54Z ajung $
"""

from zope.interface import implements

try:
    from App.class_init import InitializeClass
except ImportError:
    from Globals import InitializeClass #BBB: for Zope < 2.11.3
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.PluginIndexes.interfaces import IPluggableIndex
from Products.PluginIndexes.common.util import parseIndexRequest
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from zopyx.txng3.core.index import Index
from zopyx.txng3.core.config import *
from zopyx.txng3.core.storage import StorageException
from Products.TextIndexNG3.interfaces.textindexng import ITextIndexNG3

marker = object

def get(o, k, default=None):
    """ return a value for a given key of a dict/record 'o' """
    if isinstance(o, dict):
        return o.get(k, default)
    else:
        return getattr(o, k, default)


class TextIndexNG3(SimpleItem, PropertyManager):

    implements(ITextIndexNG3, IPluggableIndex)

    meta_type = 'TextIndexNG3'
    default_encoding = 'iso-8859-15'    # I think we don't need this anymore
    management_page_charset = 'utf-8'   # needed for several ZMI methods
    manage_options = ( {'label' : 'Index', 'action': 'manage_workspace'},
                       {'label' : 'Vocabulary', 'action' : 'vocabularyform'},
                       {'label' : 'Test', 'action' : 'queryform'},
                       {'label' : 'Converters', 'action' : 'converters'},
                       {'label' : 'Thesaurus', 'action' : 'thesaurus'},
                       {'label' : 'Adapters', 'action' : 'adapters'},
                     ) +\
                     SimpleItem.manage_options + \
                     PropertyManager.manage_options    

    query_options = ('query', 'encoding', 'parser', 'language', 'field', 'autoexpand', 'similarity_ratio', 'ranking', 'ranking_maxhits', 'thesaurus') 

    def __init__(self, id, extra, caller):
        SimpleItem.__init__(self, id, id)
        self.id = id

        # fields
        fields = [id] # default
        if get(extra, 'fields', []):
            fields = get(extra, 'fields')

        self.index = Index(fields=fields,
                           lexicon=get(extra, 'lexicon', DEFAULT_LEXICON),
                           storage=get(extra, 'storage', DEFAULT_STORAGE),
                           splitter=get(extra, 'splitter', DEFAULT_SPLITTER),
                           autoexpand=get(extra, 'autoexpand', 'off'),
                           autoexpand_limit=get(extra, 'autoexpand_limit', 4),
                           query_parser=get(extra, 'query_parser', 'txng.parsers.en'),
                           use_stemmer=get(extra, 'use_stemmer', False),
                           languages=get(extra, 'languages', ('en',)),
                           use_stopwords=bool(get(extra, 'use_stopwords')),
                           default_encoding=get(extra, 'default_encoding', DEFAULT_ENCODING),
                           use_normalizer=bool(get(extra, 'use_normalizer')),
                           dedicated_storage=bool(get(extra, 'dedicated_storage')),
                           splitter_casefolding=bool(get(extra, 'splitter_casefolding')),
                           splitter_additional_chars=get(extra, 'splitter_additional_chars', DEFAULT_ADDITIONAL_CHARS),
                           index_unknown_languages=bool(get(extra, 'index_unknown_languages', True)),
                           ranking=bool(get(extra, 'ranking')),
                           )

    def clear(self):
        """ clear the index """
        self.index.clear()

    def index_object(self, docid, obj, threshold=None):
        result = self.index.index_object(obj, docid)
        return int(result)

    def unindex_object(self, docid):
        self.index.unindex_object(docid)
        return 1

    def getIndexSourceNames(self):
        """ return indexed fields """
        return self.index.fields

    def indexSize(self):
        return len(self.index.getLexicon())

    def getEntryForObject(self, docid, default=None):
        """Get all information contained for 'docid'.

        Returns a string representing a mapping field -> list of indexed words
        for dedicated storages or a list of indexed words for shared storage.
        """
        getWord = self.index.getLexicon().getWord
        d = {}
        for field in self.index.fields:
            try:
                wids = self.index.getStorage(field).getWordIdsForDocId(docid)
            except StorageException:
                wids = ()
            words = [getWord(wid) for wid in wids]
            d[field] = words
        if not self.index.dedicated_storage:
            return repr(d[self.index.fields[0]])
        return repr(d)

    def _apply_index(self, request, cid=''):

        # parse the query options
        record = parseIndexRequest(request, self.getId(), self.query_options)
        if record.keys is None: 
            return None

        # prepare query (must be unicode string)
        query = record.keys[0]
        if not isinstance(query, unicode):
            query = unicode(query, record.get('encoding', self.index.default_encoding), 'ignore')
        if not query:
            return None

        # options
        options = {}
        for k in ('parser', 'language', 'field', 'autoexpand', 'similarity_ratio', 'thesaurus', 'ranking', 'ranking_maxhits') :
            v = getattr(record, k, marker)
            if v is not marker:
                options[k] = v

        result = self.index.search(query, **options)                                               
        ranked_resultset = result.getRankedResults()
        if ranked_resultset:
            return ranked_resultset, self.id
        else:
            return result.getDocids(), self.id

    def __len__(self):
        return len(self.index)
    numObjects = __len__

    def manage_workspace(self, REQUEST):
        """ redirect to manage since we can not override manage_workspace
            through a Five browser view
        """
        from zope.component import getMultiAdapter
        view = getMultiAdapter((self, REQUEST), name='manageform')
        return view() 


InitializeClass(TextIndexNG3)


manage_addTextIndexNG3Form = PageTemplateFile( "pt/add.pt", globals(), __name__ = 'manage_addTextIndexNG3Form')

def manage_addTextIndexNG3(self, id, extra=None, REQUEST=None, RESPONSE=None, URL3=None):
    """ Hook for the ZCatalog UI """

    return self.manage_addIndex(id, 'TextIndexNG3', extra=extra, 
                REQUEST=REQUEST, RESPONSE=RESPONSE, URL1=URL3)



###########################################################################
# TextIndexNG V 3
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

"""
TextIndexNG3 test layer

$Id: txngtest.py 2144 2009-03-15 17:41:50Z yvoschu $
"""

from Products.Five import zcml
from zope.testing.cleanup import cleanUp


class TextIndexNG3ZCMLLayer:

    @classmethod
    def setUp(cls):
        import Products.Five
        import Products.TextIndexNG3

        zcml.load_config('meta.zcml', Products.Five)
        zcml.load_config('configure.zcml', Products.Five)
        try:
            import Products.GenericSetup
            zcml.load_config('meta.zcml', Products.GenericSetup)
        except ImportError:
            pass
        zcml.load_config('configure.zcml', Products.TextIndexNG3)

    @classmethod
    def tearDown(cls):
        cleanUp()

# Derive from ZopeLite layer if available
try:
    from Testing.ZopeTestCase.layer import ZopeLite
except ImportError:
    pass # BBB: Zope < 2.11
else:
    TextIndexNG3ZCMLLayer.__bases__ = (ZopeLite,)



##########################################################################
# TextIndexNG3
# (C) 2007, ZOPYX Ltd & Co. KG, Tuebingen, Germany
##########################################################################

import os
from setuptools import setup, find_packages


CLASSIFIERS = [
    'Programming Language :: Python',
]

version_file = os.path.join('Products', 'TextIndexNG3', 'version.txt')
version = open(version_file).read().strip()

readme_file= os.path.join('Products', 'TextIndexNG3', 'doc', 'README.txt')
desc = open(readme_file).read().strip()
changes_file = os.path.join('Products', 'TextIndexNG3', 'CHANGES.txt')
changes= open(changes_file).read().strip()

long_description = desc  + '\n\nCHANGES\n===========\n\n' + changes


setup(name='Products.TextIndexNG3',
      version=version,
      license='ZPL (see LICENSE.txt)',
      author='Andreas Jung',
      author_email='info@zopyx.com',
      maintainer='Andreas Jung',
      maintainer_email='info@zopyx.com',
      classifiers=CLASSIFIERS,
      keywords='Zope2 Zope3 fulltext indexing ',
      url='http://opensource.zopyx.com/projects/TextIndexNG3',
      description='Pluggable fulltext indexing solution for Zope 2 and Zope 3',
      long_description=long_description,
      packages=['Products', 'Products.TextIndexNG3'],
      include_package_data = True,
      zip_safe=False,
      install_requires=['setuptools', 'zopyx.txng3.core', 'zopyx.txng3.ext'],
      tests_require=('zope.testing', ),
      namespace_packages=['Products'],

      )



#-*- coding: iso-8859-15 -*-

###########################################################################
# TextIndexNG V 3                
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

# some texts for test cases

de1 = """
Gleich ihr ist er Reflexionsbestimmung. Das Opfer wird uns frei machen, was
historisch den Sprachen ohnehin widerfuhr. (6) Heideggers Lehre läuft ungewollt
auf eine Exegese des schnöden Witzes "Umsonst ist der Tod", so hat bei den
existentiellen Eigentlichen selbst der Existentialismus nichts zu lachen: Je
weniger das für sich seiende Subjekt vermag; tiefinnerstes Anliegen sein
müssen, bei denen Verwaltungsmänner, als Ganzes. (7)(8) Aber die falsche Würde
hat nicht immer Unrecht, weil man tief sein will, die seiner Sprache folgten,
was er im gleichen Augenblick tut: 
"""

de2 = """
Auch Heidegger begegnete dem Nichts nur zur höheren Propädeutik des Seins.
Spätbürgerliches Denken bildet sich zurück zur nackten Selbsterhaltung, was
bloß jenem Vorteil dient. (5)(6)(7) Das sichtbar zu machen, echtes Gespräch,
welche einst dem Denken zur Emanzipation verhalf. Sie sucht das Neue nur,
daseiend, die ihrerseits wieder durch Erinnerung an Theologie mehr sein wollen,
was seinem Sinn nach daraus folgen sollte. 
"""

de3 = """
Deshalb besteht der formale Schluß von dem überdies ontologisch unangemessen
als Ausstand interpretierten Noch-nicht des Daseins auf seine Unganzheit nicht
zurecht. Wer die Warenwelt als das "An sich" akzeptiert, versteift die
triebfeindlichen Tabus der Innerlichen. (1) Unterschlagen ist, worin sie
geheimes Einverständnis erwartet, mit welchen er das Wort Mensch Emanzipation registriert,
richtiges Leben definiert durch Entscheidung schlechthin. 
"""

en1 = """
 If one examines semioticist discourse, one is faced with a choice: either
reject postpatriarchial conceptualist theory or conclude that sexual identity
has intrinsic meaning. If prepatriarchial deconstruction holds, we have to
choose between the neocultural paradigm of narrative and cultural subdialectic
theory.
"""

en2 = """
It could be said that the premise of semanticist narrative suggests that
sexuality is used to oppress minorities, but only if art is distinct from
language; if that is not the case, we can assume that the raison d'etre of the
artist is deconstruction. Lyotard promotes the use of capitalist subcultural
theory to challenge sexist perceptions of society.
"""

en3 = """
Therefore, Baudrillard's model of predialectic situationism implies that the
establishment is intrinsically responsible for capitalism, given that the
premise of the neocultural paradigm of narrative is invalid. Dahmus[1] suggests
that we have to choose between postpatriarchial conceptualist theory and
textual objectivism. 
"""

fr1 = """
Benoît XVI a inauguré aujourd'hui son pontificat avec une messe solennelle de
près de trois heures célébrée place Saint Pierre devant 350.000 fidèles, habité
par le souvenir de son prédécesseur Jean Paul II, et par la volonté d'agir pour
l'unité de tous les chrétiens. ...
"""

fr2 = """
Pour les autres pièces d'habillement, la hausse des importations varie de 63%
pour les soutiens-gorge à 413% sur les pantalons pour homme.
"""

fr3 = """
Sur les neuf catégories, sept répondent aux demandes du groupe de pression
textile européen Euratex qui avait mis en cause un "tsunami" d'importations
chinoises pour douze produits. Les autres demandent "une poursuite de
l'analyse", a dit le commissaire. En revanche, les T-shirts et un type de fil,
non mentionnés par Euratex appellent également une action, selon Bruxelles.
"""




###########################################################################
# TextIndexNG V 3
# The next generation TextIndex for Zope
#
# This software is governed by a license. See
# LICENSE.txt for the terms of this license.
###########################################################################

"""
export / import support

$Id: exportimport.py 1833 2007-03-19 11:07:57Z yvoschu $
"""

from zope.component import adapts

from Products.GenericSetup.interfaces import ISetupEnviron
from Products.GenericSetup.utils import NodeAdapterBase

from Products.TextIndexNG3.interfaces import ITextIndexNG3


class TextIndexNG3NodeAdapter(NodeAdapterBase):

    """Node im- and exporter for TextIndexNG3.
    """

    adapts(ITextIndexNG3, ISetupEnviron)

    def _exportNode(self):
        """Export the object as a DOM node.
        """
        node = self._getObjectNode('index')

        settings = self.context.index.getSettings()
        items = settings.items()
        items.sort()
        for key, value in items:
            if key == 'splitter_max_length':
                continue
            if isinstance(value, (tuple, list)):
                for item in value:
                    child = self._doc.createElement(key[:-1])
                    child.setAttribute('value', unicode(item))
                    node.appendChild(child)
            else:
                child = self._doc.createElement(key)
                child.setAttribute('value', unicode(value))
                node.appendChild(child)

        return node

    def _importNode(self, node):
        """Import the object from the DOM node.
        """
        node_infos = [ {'name': child.nodeName,
                        'value': child.getAttribute('value').encode('utf-8')}
                       for child in node.childNodes
                       if child.nodeType == child.ELEMENT_NODE ]

        settings = self.context.index.getSettings()
        new_settings = {}
        for key, value in settings.items():
            if key == 'splitter_max_length':
                continue
            if isinstance(value, (tuple, list)):
                name = key[:-1]
            else:
                name = key
            new_value = [ info['value']
                          for info in node_infos if info['name'] == name ]
            if not new_value:
                continue
            if new_value == ['None']:
                new_value = None
            elif isinstance(value, bool):
                new_value = self._convertToBoolean(new_value[0])
            elif isinstance(value, basestring) or value is None:
                new_value = new_value[0]
            elif isinstance(value, int):
                new_value = int(new_value[0])
            elif isinstance(value, tuple):
                new_value = tuple(new_value)
            if new_value != value:
                new_settings[key] = new_value
        if new_settings:
            settings.update(new_settings)
            self.context.index.__init__(**settings)

    node = property(_exportNode, _importNode)
